import { assert } from "chai";
import { DataFrame, Series, get_dummies } from "../../dist";

describe("DummyEncoder", function () {
  it("get_dummies works on Series", function () {

    const data = ["dog", "male", "female", "male", "female", "male", "dog"];
    const series = new Series(data);
    const df = get_dummies(series, { prefix: "test", prefixSeparator: "/" });

    const dfValues = [
      [1, 0, 0],
      [0, 1, 0],
      [0, 0, 1],
      [0, 1, 0],
      [0, 0, 1],
      [0, 1, 0],
      [1, 0, 0]
    ];
    const dfColumns = ['test/dog', 'test/male', 'test/female'];
    assert.deepEqual(df.values, dfValues);
    assert.deepEqual(df.columns, dfColumns);
  });
  it("get_dummies works on Series with default prefix and prefixSeperator", function () {

    const data = ["dog", "male", "female", "male", "female", "male", "dog"];
    const series = new Series(data);
    const df = get_dummies(series);

    const dfValues = [
      [1, 0, 0],
      [0, 1, 0],
      [0, 0, 1],
      [0, 1, 0],
      [0, 0, 1],
      [0, 1, 0],
      [1, 0, 0]
    ];
    const dfColumns = ['0_dog', '1_male', '2_female'];
    assert.deepEqual(df.values, dfValues);
    assert.deepEqual(df.columns, dfColumns);
  });

  it("get_dummies works on DataFrame", function () {

    const data = [[1, "dog", 1.0, "fat"], [3, "fog", 2.0, "good"], [4, "gof", 3.0, "best"]];
    const columns = ["A", "B", "C", "d"];
    const df = new DataFrame(data, { columns: columns });

    const df1 = get_dummies(df, { prefixSeparator: ["_", "#"], columns: ["A", "d"], prefix: "test" });
    const expectedColumns = ['B', 'C', 'test_1', 'test_3', 'test_4', 'test#fat', 'test#good', 'test#best'];
    const expected = [['dog', 1.0, 1, 0, 0, 1, 0, 0],
      ['fog', 2.0, 0, 1, 0, 0, 1, 0],
      ['gof', 3.0, 0, 0, 1, 0, 0, 1]];
    assert.deepEqual(df1.values, expected);
    assert.deepEqual(df1.columns, expectedColumns);

  });
  it("Throw error if the prefix specified is not equal to the column specified", function () {

    const data = [[1, "dog", 1.0, "fat"], [3, "fog", 2.0, "good"], [4, "gof", 3.0, "best"]];
    const columns = ["A", "B", "C", "d"];
    const df = new DataFrame(data, { columns: columns });

    assert.throws(function () { get_dummies(df, { prefix: ["fg"], prefixSeparator: "_", columns: ["A", "d"] }); }, Error,
      `ParamError: prefix and data array must be of the same length. If you need to use the same prefix, then pass a string param instead. e.g {prefix: "fg"}`);

  });
  it("replace column sepecified with prefix", function () {

    const data = [[1, "dog", 1.0, "fat"], [3, "fog", 2.0, "good"], [4, "gof", 3.0, "best"]];
    const columns = ["A", "B", "C", "d"];
    const df = new DataFrame(data, { columns: columns });

    const df1 = get_dummies(df, { prefix: ["F", "G"], prefixSeparator: "_", columns: ["A", "d"] });
    const expectedColumns = [
      'B', 'C',
      'F_1', 'F_3',
      'F_4', 'G_fat',
      'G_good', 'G_best'
    ];

    const expected = [['dog', 1.0, 1, 0, 0, 1, 0, 0],
      ['fog', 2.0, 0, 1, 0, 0, 1, 0],
      ['gof', 3.0, 0, 0, 1, 0, 0, 1]];

    assert.deepEqual(df1.values, expected);
    assert.deepEqual(df1.columns, expectedColumns);

  });

  it("get_dummies auto infers and encode columns with string dtype", function () {

    const data = [[1, "dog", 1.0, "fat"], [3, "fog", 2.0, "good"], [4, "gof", 3.0, "best"]];
    const columns = ["A", "B", "C", "d"];
    const df = new DataFrame(data, { columns: columns });

    const df1 = get_dummies(df, { prefixSeparator: "_" });
    const expectedColumns = [
      'A', 'C',
      'B_dog', 'B_fog',
      'B_gof', 'd_fat',
      'd_good', 'd_best'
    ];
    const expected = [
      [
        1, 1, 1, 0,
        0, 1, 0, 0
      ],
      [
        3, 2, 0, 1,
        0, 0, 1, 0
      ],
      [
        4, 3, 0, 0,
        1, 0, 0, 1
      ]
    ];
    assert.deepEqual(df1.values, expected);
    assert.deepEqual(df1.columns, expectedColumns);

  });

  it("should one hot encode all other columns", function () {

    const data = [[1, "dog", 1.0, "fat"], [3, "fog", 2.0, "good"], [4, "gof", 3.0, "best"]];
    const columns = ["A", "B", "C", "d"];
    const df = new DataFrame(data, { columns: columns });
    const rslt = [
      [1, 'dog', 1, 1, 0, 0],
      [3, 'fog', 2, 0, 1, 0],
      [4, 'gof', 3, 0, 0, 1]
    ];

    assert.deepEqual(get_dummies(df, { columns: ["d"] }).values, rslt);

  });
});
